﻿using System;
using MonitorService;
using MonitorService.Impl;
using Ninject;
using NUnit.Framework;
using Rhino.Mocks;

namespace UnitTestApp
{
    [TestFixture]
    public class MonitorServiceTest_Auto
    {
        public ConfigFileMonitor CurrentFileMonitor
        {
            get;
            set;
        }

        [SetUp]
        public void SetUp()
        {
            using (var kernel = new StandardKernel(new ServiceModule()))
            {
                this.CurrentFileMonitor = kernel.Get<ConfigFileMonitor>();
            }
        }

        //[Test]
        //public void FileMonitor_Inject_GetInstance()
        //{
        //    Assert.IsNotNull(CurrentFileMonitor, "ConfigFileMonitor is not initialized");

        //    var mock = new Rhino.Mocks.MockRepository();
        //    var logService = mock.Stub<ILogService>();
        //    var emailService = mock.DynamicMock<IEmailService>();
        //    CurrentFileMonitor.LogService = logService;
        //    CurrentFileMonitor.EmailService = emailService;

        //    Assert.IsNotNull(CurrentFileMonitor.LogService, "ConfigFileMonitor stub LogService is not initialized");
        //    Assert.IsNotNull(CurrentFileMonitor.EmailService, "ConfigFileMonitor mock EmailService is not initialized");
        //}

        //[Test]
        //public void FileMonitor_LogService_Stub_GetInstance()
        //{
        //    var mock = new Rhino.Mocks.MockRepository();
        //    var logService = mock.Stub<ILogService>();
        //    CurrentFileMonitor.LogService = logService;
        //    Assert.IsNotNull(CurrentFileMonitor.LogService, "ConfigFileMonitor stub LogService is not initialized");
        //}

        //[Test]
        //public void FileMonitor_EmailService_Mock_GetInstance()
        //{
        //    var mock = new Rhino.Mocks.MockRepository();
        //    var emailService = mock.DynamicMock<IEmailService>();
        //    CurrentFileMonitor.EmailService = emailService;
        //    Assert.IsNotNull(CurrentFileMonitor.EmailService, "ConfigFileMonitor mock EmailService is not initialized");
        //}

        /// <summary>
        /// 使用录制-回放模型测试示例
        /// </summary>
        [Test]
        public void RhinoMocks_Analyze_WebServiceThrows_SendEmail()
        {
            var mock = new Rhino.Mocks.MockRepository();
            var logService = mock.Stub<ILogService>();
            var emailService = mock.DynamicMock<IEmailService>();
            using (mock.Record())
            {
                logService.AppendLog("input filename(abc.txt) is too short");
                LastCall.Throw(new NotImplementedException("fake exception")); //记录指定日志，抛出异常
            }
            CurrentFileMonitor.LogService = logService;
            CurrentFileMonitor.EmailService = emailService;

            var shortFileName = "abc.txt";
            CurrentFileMonitor.Analyze(shortFileName);

            mock.Verify(emailService);//VerifyAll()或者Verify(stub)不会对桩对象做任何断言，它们只适用于模拟对象
        }

        /// <summary>
        /// 设置-操作-断言模型测试示例 （Arrange-Act-Assert AAA）
        /// </summary>
        [Test]
        public void RhinoMocks_Analyze_WebServiceThrows_SendEmail_AAA()
        {
            var mock = new Rhino.Mocks.MockRepository();
            var logService = mock.Stub<ILogService>();
            logService.Expect(svc => svc.AppendLog("input filename(abc.txt) is too short"))
                      .Throw(new NotImplementedException("fake exception")); //记录指定日志，抛出异常

            var emailService = mock.DynamicMock<IEmailService>();
            CurrentFileMonitor.LogService = logService;
            CurrentFileMonitor.EmailService = emailService;

            mock.ReplayAll(); //移到操作模式

            var shortFileName = "abc.txt";
            CurrentFileMonitor.Analyze(shortFileName);

            //使用Rhino Mocks来断言
            emailService.AssertWasCalled(m => m.SendEmail("jeffwong@cnblogs.com", "filename check", "fake exception"));//成功
            //emailService.AssertWasCalled(m => m.SendEmail("it@cnblogs.com", "filename check", "fake exception"));//失败
        }

        [TearDown]
        public void TearDown()
        {
            this.CurrentFileMonitor = null;
        }

    }
}